@@ -15,20 +15,36 @@ class Job {
   File file;
   String name;
   SecretKeySpec sks;
+  boolean has_password;
 
   Job (MODE m, File f, String n, String p)
   {
     mode = m;
-    file = f;
+    try {
+      file = new File(f.getCanonicalPath());
+    } catch (Exception e) {
+      file = f;
+    }
     name = n;
 
-    sks = new SecretKeySpec(p.substring(0, 16).getBytes(), "AES");
+    if (p.length() > 0) {
+      sks = new SecretKeySpec(AES_align(p).substring(0, 16).getBytes(), "AES");
+      has_password = true;
+    }
+    else
+      has_password = false;
   }
 
+  static String AES_align (String s)
+  {
+    return s.concat("                ".substring(0, 16 - (s.length() % 16)));
+  }
 
   MODE getMode () { return mode; }
   File getFile () { return file; }
   String getName () { return name; }
+  Boolean hasPassword () { return has_password; }
+
   Cipher getCipher () throws Exception
   {
     Cipher cipher = Cipher.getInstance("AES");
@@ -75,9 +91,38 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
   JPasswordField pwdField;
   DefaultListModel taskListModel, repositoryListModel;
   Thread worker;
+  long currentSize, bytesSoFar, startTime;
+  JFileChooser importChooser, exportChooser;
+  boolean brokenJFileChooser;
 
   lechunk()
   {
+    brokenJFileChooser = System.getProperty("os.name").equals("Mac OS X");
+    exportChooser = new JFileChooser();
+    if (brokenJFileChooser) {
+      JPanel jp = new JPanel();
+      jp.setLayout(new BoxLayout(jp, BoxLayout.Y_AXIS));
+      String[] msg = {
+        "You are running a",
+        "broken operating system.",
+        "Do not select the",
+        "destination directory.",
+        "Change into it!"
+      };
+      for (int i = 0; i < msg.length; i++) {
+        JLabel jl = new JLabel(msg[i]);
+        jl.setAlignmentX(Component.CENTER_ALIGNMENT);
+        jp.add(jl);
+      }
+      exportChooser.setAccessory(jp);
+    }
+    exportChooser.setMultiSelectionEnabled(false);
+    exportChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
+
+    importChooser = new JFileChooser();
+    importChooser.setMultiSelectionEnabled(true);
+    importChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
+
     JPanel mainPanel = new JPanel();
     mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.LINE_AXIS));
     JButton button = new JButton("Import");
@@ -186,11 +231,6 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
     updateRepositoryList();
   }
 
-  String AES_align (String s)
-  {
-    return s.concat("                ".substring(0, 16 - (s.length() % 16)));
-  }
-
   void wrkDirCleanup (File wrkDir)
   {
     File dstFiles[] = wrkDir.listFiles();
@@ -209,20 +249,21 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
   boolean importChunk (FileInputStream src, OutputStream dst, String name) throws IOException
   {
     byte buffer[] = new byte[65536];
-    int readTotal, readBytes;
-    long startTime = Calendar.getInstance().getTimeInMillis() / 1000;
+    long readTotal;
+    int readBytes;
 
     for (readTotal = 0; readTotal < CHUNK_SIZE; readTotal += readBytes) {
       readBytes = src.read(buffer);
       if (readBytes == -1) return false;
       dst.write(buffer, 0, readBytes);
-      progressBar.setValue(progressBar.getValue() + readBytes);
+      bytesSoFar += readBytes;
+      progressBar.setValue((int)(10000 * bytesSoFar/currentSize));
       progressBar.setString("Importing " + name + ": " +
-                            (int) (100 * progressBar.getPercentComplete()) +
+                            (int)(100 * bytesSoFar/currentSize) +
                             "%");
       long now = Calendar.getInstance().getTimeInMillis() / 1000;
       if (now > startTime) {
-        long speed = (readTotal / 1024) / (now - startTime);
+        long speed = (bytesSoFar / 1024) / (now - startTime);
         progressBar.setToolTipText(new Long(speed).toString() + " kB/s");
       }
     }
@@ -260,15 +301,16 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
     progressBar.setStringPainted(true);
 
     try {
-      CipherOutputStream dst;
+      OutputStream dst;
       FileInputStream src = new FileInputStream(srcFile);
-      int size = src.available();
-      byte bsize[] = AES_align(new Integer(size).toString()).getBytes("US-ASCII");
-      progressBar.setMaximum(size);
+      bytesSoFar = 0;
+      currentSize = srcFile.length();
+      startTime = Calendar.getInstance().getTimeInMillis() / 1000;
+      byte bsize[] = Job.AES_align(new Long(currentSize).toString()).getBytes("US-ASCII");
+      progressBar.setMaximum(10000);
       do {
-        dst = new CipherOutputStream(
-          new FileOutputStream(new File(wrkDir, new Formatter().format("%016d", chunk).toString())),
-          job.getCipher());
+        dst = new FileOutputStream(new File(wrkDir, new Formatter().format("%016d", chunk).toString()));
+        if (job.hasPassword()) dst = new CipherOutputStream(dst, job.getCipher());
         srcHasSome = importChunk(src, dst, name);
         dst.close();
         ++chunk;
@@ -277,9 +319,8 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
       FileOutputStream dst2 = new FileOutputStream(new File(REPOSITORY_DIR, name + ".md5"));
       dst2.write(bsize);
       dst2.close();
-      dst = new CipherOutputStream(
-        new FileOutputStream(new File(REPOSITORY_DIR, name + ".size")),
-        job.getCipher());
+      dst = new FileOutputStream(new File(REPOSITORY_DIR, name + ".size"));
+      if (job.hasPassword()) dst = new CipherOutputStream(dst, job.getCipher());
       dst.write(bsize);
       dst.close();
     } catch (Exception e) {
@@ -292,24 +333,23 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
     progressBar.setMaximum(0);
   }
 
-  void exportChunk (File srcFile, FileOutputStream dst, String name, Cipher cipher) throws Exception
+  void exportChunk (InputStream src, FileOutputStream dst, String name) throws Exception
   {
     byte buffer[] = new byte[65536];
-    int readTotal = 0, readBytes;
-    long startTime = Calendar.getInstance().getTimeInMillis() / 1000;
-
-    CipherInputStream src = new CipherInputStream(new FileInputStream(srcFile), cipher);
+    long readTotal = 0;
+    int readBytes;
 
     while ((readBytes = src.read(buffer)) != -1) {
       dst.write(buffer, 0, readBytes);
       readTotal += readBytes;
-      progressBar.setValue(progressBar.getValue() + readBytes);
+      bytesSoFar += readBytes;
+      progressBar.setValue((int)(10000 * bytesSoFar/currentSize));
       progressBar.setString("Exporting " + name + ": " +
-                            (int) (100 * progressBar.getPercentComplete()) +
+                            (int)(100 * bytesSoFar/currentSize) +
                             "%");
       long now = Calendar.getInstance().getTimeInMillis() / 1000;
       if (now > startTime) {
-        long speed = (readTotal / 1024) / (now - startTime);
+        long speed = (bytesSoFar / 1024) / (now - startTime);
         progressBar.setToolTipText(new Long(speed).toString() + " kB/s");
       }
     }
@@ -335,26 +375,33 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
     if (dstFile.exists()) if (!exportOverwrite(dir, file)) return;
 
     try {
-      FileInputStream src2 = new FileInputStream(new File(REPOSITORY_DIR, file + ".md5"));
+      File f = new File(REPOSITORY_DIR, file + ".md5");
+      FileInputStream src2 = new FileInputStream(f);
+      startTime = Calendar.getInstance().getTimeInMillis() / 1000;
       byte tmp2[] = new byte[src2.available()];
       byte tmp[] = new byte[src2.available()];
       src2.read(tmp2);
       src2.close();
-      CipherInputStream src = new CipherInputStream(
-        new FileInputStream(new File(REPOSITORY_DIR, file + ".size")),
-        job.getCipher());
+      InputStream src = new FileInputStream(new File(REPOSITORY_DIR, file + ".size"));
+      if (job.hasPassword()) src = new CipherInputStream(src, job.getCipher());
       src.read(tmp);
       src.close();
       if (!new String(tmp).equals(new String(tmp2))) {
         JOptionPane.showMessageDialog(null, "Wrong password!", file, JOptionPane.ERROR_MESSAGE);
         return;
       }
+      bytesSoFar = 0;
+      currentSize = new Long(new String(tmp).trim()).longValue();
       progressBar.setValue(0);
       progressBar.setStringPainted(true);
-      progressBar.setMaximum(new Integer(new String(tmp2, "US-ASCII").trim()).intValue());
+      progressBar.setMaximum(10000);
       FileOutputStream dst = new FileOutputStream(dstFile);
       File srcFiles[] = new File(REPOSITORY_DIR, file).listFiles();
-      for (int i = 0; i < srcFiles.length; ++i) exportChunk(srcFiles[i], dst, file, job.getCipher());
+      for (int i = 0; i < srcFiles.length; ++i) {
+        src = new FileInputStream(srcFiles[i]);
+        if (job.hasPassword()) src = new CipherInputStream(src, job.getCipher());
+        exportChunk(src, dst, file);
+      }
       dst.close();
     } catch (Exception e) {
       JOptionPane.showMessageDialog(null, e.toString(), file, JOptionPane.ERROR_MESSAGE);
@@ -414,26 +461,20 @@ class lechunk implements ActionListener, Runnable, ListSelectionListener, Filena
 
   void actionImport ()
   {
-    JFileChooser chooser = new JFileChooser();
-    chooser.setMultiSelectionEnabled(true);
-    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
-    if (chooser.showOpenDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
-      File files[] = chooser.getSelectedFiles();
+    if (importChooser.showOpenDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
+      File files[] = importChooser.getSelectedFiles();
       for (int i = 0; i < files.length; ++i)
-        updateTaskList(TASKLIST_MODE.PUSH, new Job(Job.MODE.IMPORT, files[i], null, AES_align(new String(pwdField.getPassword()))));
+        updateTaskList(TASKLIST_MODE.PUSH, new Job(Job.MODE.IMPORT, files[i], null, new String(pwdField.getPassword())));
     }
   }
 
   void actionExport ()
   {
-    JFileChooser chooser = new JFileChooser();
-    chooser.setMultiSelectionEnabled(false);
-    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
-    if (chooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
-      File dir = chooser.getSelectedFile();
+    if (exportChooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
+      File dir = (brokenJFileChooser ? exportChooser.getCurrentDirectory() : exportChooser.getSelectedFile());
       String files[] = getSelectedRepositoryFiles();
       for (int i = 0; i < files.length; ++i)
-        updateTaskList(TASKLIST_MODE.PUSH, new Job(Job.MODE.EXPORT, dir, files[i], AES_align(new String(pwdField.getPassword()))));
+        updateTaskList(TASKLIST_MODE.PUSH, new Job(Job.MODE.EXPORT, dir, files[i], new String(pwdField.getPassword())));
     }
   }
 
